// csv: comma separated values.
import { select, csv, scaleLinear,
        max, scaleBand, axisLeft, axisBottom } from 'd3';
// Define svg canvas:
const svg = select('svg')
  .style('background-color', 'lightgray');
// svg defined in index.html:
const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  // Value Accessors:
  const xValue = d => d.population;
  const yValue = d => d.country;
  // d3 Margin Convention:
  const margin = { top:20,right:40,bottom:20,left:100 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()  // Used for Quantitative Attributes.
  	.domain([0, max(data, xValue)])  // Data Space using population.
  	.range([0, innerWidth]);  // Screen Space.

  const yScale = scaleBand()  // Used for Ordinal Attributes.
  	.domain(data.map(yValue))  // Data Space using countries.
  	.range([0, innerHeight])  // Screen Space.
  	.padding(0.1);  // Space between rectangles.
  
  // Define group element:
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left},${margin.top})`)
  
    g.append('g').call(axisLeft(yScale)); // xAxis defined in call argument.
    g.append('g').call(axisBottom(xScale))  // yAxis defined in call argument.
  		.attr('transform', `translate(0, ${innerHeight})`); // xAxis numbers: x=0, y=innerHeight.

  // Data join DOM elements to data set.
  g.selectAll('rect')
  	.data(data)
  	.enter().append('rect')
  		.attr('y', d => yScale(yValue(d)))
  		.attr('width', d => xScale(xValue(d)))
  		.attr('height', yScale.bandwidth()) // Bandwidth is calculated rectangle width.
}
// Representing a data table using javaScript:
// Load data.csv which returns Promise with resolve callback, with the arguement data.
csv('data.csv').then( (data) => {
  // Function body:
  // Convert population from a string to a number using Unary '+' operator.
  data.forEach(d => {
    d.population = +d.population * 1000
  });
  // Executable callback function:
  render(data);
});